#!/usr/bin/perl -I/sw/lib/perl5/5.8.6

use lib "/path/to/your/gconfig_pm";

package graffiti; 

use strict; 
use DBI; 
use gconfig; 
use Digest::MD5 qw(md5_hex); 

my $dbh = DBI->connect(gconfig->dbi);

use XMLRPC::Transport::HTTP; 
my $server = XMLRPC::Transport::HTTP::CGI
   -> dispatch_to('graffiti')
   -> handle
;

#################### Functions used by the script ####################

# Clean a url for SQL use
sub scrub {
	my ($self, $url) = @_; 
	$$url =~ s/^https/http/; 	# If it's an URL
	$$url =~ s/^http\:\/\/www\./http\:\/\//;
	$$url =~ s/\/$//; 
	$$url = $dbh->quote($$url);
}

# Takes a username/pass, returns user ID on success, 
# 0 for "wrong pass", or -1 for anonymous
sub login {
	my ( $self, $user, $pass) = @_; 
	return -1 unless ($dbh);  # Could not establish SQL connection?
	return -1 unless ($user); # Anonymous user
	$pass = md5_hex($pass);
	$user = $dbh->quote($user);


	my $sth = $dbh->prepare(qq[select id,pass from guser where name=$user;]); $sth->execute(); 

	my @result = $sth->fetchrow_array;
	$sth->finish;

	if (@result) { # If we found the user
		my ($id, $spass) = @result; 
		return $id if ($pass eq $spass);
	} else {				# If we have to create a user
		my $tth = $dbh->prepare(qq[insert into guser(name,pass,created) values ($user,"$pass",now());]); $tth->execute();
		my $id = $dbh->{'mysql_insertid'};
		$tth->finish(); 
		return $id;
	}
	
	return 0;
}

################# Functions called by XML-RPC clients ####################

# Return an array of all all "notes" for a given URL.
# Args: An URL, a note quantity maximum, a note score minimum
sub queso {
	my ($self, $url, $limit, $thresh) = @_;
	scrub($self, \$url); $limit = int($limit); 
	$thresh = int($thresh);
	$thresh = $thresh ? "" : " and gnote.score >= $thresh"; 
	$limit = $limit < 0 ? "" : " order by score desc limit $limit"; 
	my @found; 

	return [] unless ($dbh);  # Could not establish SQL connection?

	my $sth = $dbh->prepare(qq[select guser.name, gnote.content, gnote.id, guser.id, gnote.x, gnote.y, gnote.score from (gnote, guser) where gnote.url=$url and guser.id=gnote.owner$thresh$limit;]); $sth->execute();
	return [] if ($url =~ /foxnews|cnn|msnbc|accuweather|weather\.com|news\.yahoo|abcnews|chron\.com/);

	while (my @result = $sth->fetchrow_array) {
		push(@found, { owner => "$result[0]", content => $result[1], id => "$result[2]", nid => "$result[3]", x => $result[4], y => $result[5], score => $result[6] }); 
	} 

	$sth->finish;
	return @found ? [@found] : [];
} 

# Insert a note and return a structure representing it.
# Args: url, message, username and password
sub post {
	my ($self, $url, $message, $user, $pass) = @_;
	my $id = login($self,$user,$pass); 
	my $smessage = $dbh->quote($message); # sql cleaned version

	return [{error => "Graffiti error: Sticky server cannot contact database."}] unless ($dbh); 
	return [{error => "Graffiti error: You need to enter a username and password to post notes. Please edit your preferences for the Graffiti plugin. You can choose any username and password you like.", needprefs => 1}] if ($id < 0);
	return [{error => "Graffiti error: Sorry, either that username is taken or you entered the wrong password."}] if ($id == 0); 

        scrub($self, \$url); 

	return [{error =>"Graffiti error: No message entered."}] unless ($message =~ /\S/);
	
	my ($x,$y) = (int(rand(100)), int(rand(100)));
	my $sth = $dbh->prepare(qq[insert into gnote(url,content,owner,x,y,ip,created) values ($url,$smessage,$id,$x,$y,"$ENV{REMOTE_ADDR}",now());]); $sth->execute(); 
        my $nid = $dbh->{'mysql_insertid'}; 
	$sth->finish;
        return [{owner => $user, content => $message, nid => $id, id => $nid, x => $x, y => $y, score => 0 }];
}

# Increase the score for a note
# Args: message id, username and password (optional)
sub plus {
	my ($self, $message, $user, $pass) = @_;
	my $id = login($self,$user,$pass); $message = int($message);
	my $sth = $dbh->prepare(qq[insert into glog(act,user,note,ip,created) values ("PLUS",$id,$message,"$ENV{REMOTE_ADDR}",now());]); $sth->execute(); 
        $sth->finish; 
	my $sth = $dbh->prepare(qq[update gnote set score=score+1 where id=$message;]); $sth->execute();
        $sth->finish; 
}

# Decrease the score for a note
# Args: message id, username and password (optional) 
sub minus {
	my ($self, $message, $user, $pass) = @_;
        my $id = login($self,$user,$pass); $message = int($message);
        my $sth = $dbh->prepare(qq[insert into glog(act,user,note,ip,created) values ("MINUS",$id,$message,"$ENV{REMOTE_ADDR}",now());]); $sth->execute();
        $sth->finish; 
        my $sth = $dbh->prepare(qq[update gnote set score=score-1 where id=$message;]); $sth->execute();
        $sth->finish; 
}

# Decrease the score for a note and possibly delete it
# Args: message id, username and password (optional) 
sub kill {
	my ($self, $message, $user, $pass) = @_;
        my $id = login($self,$user,$pass); $message = int($message);
        my $sth = $dbh->prepare(qq[insert into glog(act,user,note,ip,created) values ("KILL",$id,$message,"$ENV{REMOTE_ADDR}",now());]); $sth->execute();
        $sth->finish; 
        my $sth = $dbh->prepare(qq[update gnote set score=score-1 where id=$message;]); $sth->execute();
        $sth->finish; 
	my $sth = $dbh->prepare(qq[delete from gnote where id=$message and owner=$id;]); $sth->execute();
        $sth->finish; 
}

# Move a note on the screen
# Args: message id, username and password (optional), x and y coords
sub move {
	my ($self, $message, $user, $pass, $x, $y) = @_;
        my $id = login($self,$user,$pass); $message = int($message);
	$x = int($x); $y = int($y); 

        my $sth = $dbh->prepare(qq[insert into glog(act,user,note,ip,created) values ("MOVE",$id,$message,"$ENV{REMOTE_ADDR}",now());]); $sth->execute();
        $sth->finish; 
        my $sth = $dbh->prepare(qq[update gnote set x=$x, y=$y where id=$message;]); $sth->execute();
        $sth->finish;

}

# This... doesn't really do anything
sub sandwich {
	return "SANDWICH!!!!";
} 
